# ruff: noqa: UP006 UP007
import typing as ta

from omlish.lite.inject import InjectorBindingOrBindings
from omlish.lite.inject import InjectorBindings
from omlish.lite.inject import inj

from .config import RemoteConfig
from .connection import PyremoteRemoteExecutionConnector
from .connection import RemoteExecutionConnector
from .payload import RemoteExecutionPayloadFile
from .spawning import RemoteSpawning
from .spawning import SubprocessRemoteSpawning


def bind_remote(
        *,
        remote_config: RemoteConfig,
) -> InjectorBindings:
    lst: ta.List[InjectorBindingOrBindings] = [
        inj.bind(remote_config),

        inj.bind(SubprocessRemoteSpawning, singleton=True),
        inj.bind(RemoteSpawning, to_key=SubprocessRemoteSpawning),

        inj.bind(PyremoteRemoteExecutionConnector, singleton=True),
        inj.bind(RemoteExecutionConnector, to_key=PyremoteRemoteExecutionConnector),
    ]

    if (pf := remote_config.payload_file) is not None:
        lst.append(inj.bind(pf, to_key=RemoteExecutionPayloadFile))

    return inj.as_bindings(*lst)



import dataclasses as dc

from .config import MainConfig
from .remote.config import RemoteConfig


@dc.dataclass(frozen=True)
class MainBootstrap:
    main_config: MainConfig = MainConfig()

    remote_config: RemoteConfig = RemoteConfig()



# ruff: noqa: UP006 UP007
"""
~deploy
  deploy.pid (flock)
  /app
    /<appspec> - shallow clone
  /conf
    /env
      <appspec>.env
    /nginx
      <appspec>.conf
    /supervisor
      <appspec>.conf
  /venv
    /<appspec>

?
  /logs
    /wrmsr--omlish--<spec>

spec = <name>--<rev>--<when>

==

for dn in [
    'app',
    'conf',
    'conf/env',
    'conf/nginx',
    'conf/supervisor',
    'venv',
]:
"""
import abc
import dataclasses as dc
import os.path
import typing as ta

from omlish.lite.check import check


DeployPathKind = ta.Literal['dir', 'file']  # ta.TypeAlias


##


DEPLOY_PATH_SPEC_PLACEHOLDER = '@'


@dc.dataclass(frozen=True)
class DeployPathPart(abc.ABC):  # noqa
    @property
    @abc.abstractmethod
    def kind(self) -> DeployPathKind:
        raise NotImplementedError

    @abc.abstractmethod
    def render(self) -> str:
        raise NotImplementedError


#


class DeployPathDir(DeployPathPart, abc.ABC):
    @property
    def kind(self) -> DeployPathKind:
        return 'dir'

    @classmethod
    def parse(cls, s: str) -> 'DeployPathDir':
        if DEPLOY_PATH_SPEC_PLACEHOLDER in s:
            check.equal(s, DEPLOY_PATH_SPEC_PLACEHOLDER)
            return SpecDeployPathDir()
        else:
            return ConstDeployPathDir(s)


class DeployPathFile(DeployPathPart, abc.ABC):
    @property
    def kind(self) -> DeployPathKind:
        return 'file'

    @classmethod
    def parse(cls, s: str) -> 'DeployPathFile':
        if DEPLOY_PATH_SPEC_PLACEHOLDER in s:
            check.equal(s[0], DEPLOY_PATH_SPEC_PLACEHOLDER)
            return SpecDeployPathFile(s[1:])
        else:
            return ConstDeployPathFile(s)


#


@dc.dataclass(frozen=True)
class ConstDeployPathPart(DeployPathPart, abc.ABC):
    name: str

    def __post_init__(self) -> None:
        check.non_empty_str(self.name)
        check.not_in('/', self.name)
        check.not_in(DEPLOY_PATH_SPEC_PLACEHOLDER, self.name)

    def render(self) -> str:
        return self.name


class ConstDeployPathDir(ConstDeployPathPart, DeployPathDir):
    pass


class ConstDeployPathFile(ConstDeployPathPart, DeployPathFile):
    pass


#


class SpecDeployPathPart(DeployPathPart, abc.ABC):
    pass


class SpecDeployPathDir(SpecDeployPathPart, DeployPathDir):
    def render(self) -> str:
        return DEPLOY_PATH_SPEC_PLACEHOLDER


@dc.dataclass(frozen=True)
class SpecDeployPathFile(SpecDeployPathPart, DeployPathFile):
    suffix: str

    def __post_init__(self) -> None:
        check.non_empty_str(self.suffix)
        check.not_in('/', self.suffix)
        check.not_in(DEPLOY_PATH_SPEC_PLACEHOLDER, self.suffix)

    def render(self) -> str:
        return DEPLOY_PATH_SPEC_PLACEHOLDER + self.suffix


##


@dc.dataclass(frozen=True)
class DeployPath:
    parts: ta.Sequence[DeployPathPart]

    def __post_init__(self) -> None:
        check.not_empty(self.parts)
        for p in self.parts[:-1]:
            check.equal(p.kind, 'dir')

    @property
    def kind(self) -> ta.Literal['file', 'dir']:
        return self.parts[-1].kind

    def render(self) -> str:
        return os.path.join(  # noqa
            *[p.render() for p in self.parts],
            *([''] if self.kind == 'dir' else []),
        )

    @classmethod
    def parse(cls, s: str) -> 'DeployPath':
        tail_parse: ta.Callable[[str], DeployPathPart]
        if s.endswith('/'):
            tail_parse = DeployPathDir.parse
            s = s[:-1]
        else:
            tail_parse = DeployPathFile.parse
        ps = check.non_empty_str(s).split('/')
        return cls([
            *([DeployPathDir.parse(p) for p in ps[:-1]] if len(ps) > 1 else []),
            tail_parse(ps[-1]),
        ])



#!/usr/bin/env python3
# @omlish-amalg ../scripts/manage.py
# ruff: noqa: UP006 UP007
"""
manage.py -s 'docker run -i python:3.12'
manage.py -s 'ssh -i /foo/bar.pem foo@bar.baz' -q --python=python3.8
"""
import asyncio
import contextlib
import json
import typing as ta

from omlish.argparse.cli import ArgparseCli
from omlish.argparse.cli import argparse_arg
from omlish.argparse.cli import argparse_command
from omlish.lite.logs import log  # noqa
from omlish.lite.marshal import ObjMarshalerManager
from omlish.lite.marshal import ObjMarshalOptions
from omlish.lite.pycharm import PycharmRemoteDebug

from .bootstrap import MainBootstrap
from .bootstrap_ import main_bootstrap
from .commands.base import Command
from .commands.base import CommandExecutor
from .commands.execution import LocalCommandExecutor
from .config import MainConfig
from .remote.config import RemoteConfig
from .remote.connection import RemoteExecutionConnector
from .remote.spawning import RemoteSpawning


class MainCli(ArgparseCli):
    @argparse_command(
        argparse_arg('--payload-file'),

        argparse_arg('-s', '--shell'),
        argparse_arg('-q', '--shell-quote', action='store_true'),
        argparse_arg('--python', default='python3'),

        argparse_arg('--pycharm-debug-port', type=int),
        argparse_arg('--pycharm-debug-host'),
        argparse_arg('--pycharm-debug-version'),

        argparse_arg('--remote-timebomb-delay-s', type=float),

        argparse_arg('--debug', action='store_true'),

        argparse_arg('--local', action='store_true'),

        argparse_arg('command', nargs='+'),
    )
    def run(self) -> None:
        asyncio.run(self._async_run())

    async def _async_run(self) -> None:
        bs = MainBootstrap(
            main_config=MainConfig(
                log_level='DEBUG' if self.args.debug else 'INFO',

                debug=bool(self.args.debug),
            ),

            remote_config=RemoteConfig(
                payload_file=self.args.payload_file,  # noqa

                pycharm_remote_debug=PycharmRemoteDebug(
                    port=self.args.pycharm_debug_port,
                    **(dict(host=self.args.pycharm_debug_host) if self.args.pycharm_debug_host is not None else {}),
                    install_version=self.args.pycharm_debug_version,
                ) if self.args.pycharm_debug_port is not None else None,

                timebomb_delay_s=self.args.remote_timebomb_delay_s,
            ),
        )

        #

        injector = main_bootstrap(
            bs,
        )

        #

        msh = injector[ObjMarshalerManager]

        cmds: ta.List[Command] = []
        cmd: Command
        for c in self.args.command:
            if not c.startswith('{'):
                c = json.dumps({c: {}})
            cmd = msh.unmarshal_obj(json.loads(c), Command)
            cmds.append(cmd)

        #

        async with contextlib.AsyncExitStack() as es:
            ce: CommandExecutor

            if self.args.local:
                ce = injector[LocalCommandExecutor]

            else:
                tgt = RemoteSpawning.Target(
                    shell=self.args.shell,
                    shell_quote=self.args.shell_quote,
                    python=self.args.python,
                )

                ce = await es.enter_async_context(injector[RemoteExecutionConnector].connect(tgt, bs))  # noqa

            async def run_command(cmd: Command) -> None:
                res = await ce.try_execute(
                    cmd,
                    log=log,
                    omit_exc_object=True,
                )

                print(msh.marshal_obj(res, opts=ObjMarshalOptions(raw_bytes=True)))

            await asyncio.gather(*[
                run_command(cmd)
                for cmd in cmds
            ])


def _main() -> None:
    MainCli().call_and_exit()


if __name__ == '__main__':
    _main()



# ruff: noqa: UP006 UP007
import typing as ta

from omlish.lite.inject import InjectorBindingOrBindings
from omlish.lite.inject import InjectorBindings
from omlish.lite.inject import inj

from ..commands.inject import bind_command
from .command import DeployCommand
from .command import DeployCommandExecutor


def bind_deploy(
) -> InjectorBindings:
    lst: ta.List[InjectorBindingOrBindings] = [
        bind_command(DeployCommand, DeployCommandExecutor),
    ]

    return inj.as_bindings(*lst)
